build_defs_file = './ext/build_defaults.yaml'
if File.exist?(build_defs_file)
  begin
    require 'yaml'
    @build_defaults ||= YAML.load_file(build_defs_file)
  rescue Exception => e
    STDERR.puts "Unable to load yaml from #{build_defs_file}:"
    STDERR.puts e
  end
  @packaging_url  = @build_defaults['packaging_url']
  @packaging_repo = @build_defaults['packaging_repo']
  raise "Could not find packaging url in #{build_defs_file}" if @packaging_url.nil?
  raise "Could not find packaging repo in #{build_defs_file}" if @packaging_repo.nil?

  namespace :package do
    desc "Bootstrap packaging automation, e.g. clone into packaging repo"
    task :bootstrap do
      if File.exist?("./ext/#{@packaging_repo}")
        puts "It looks like you already have ext/#{@packaging_repo}. If you don't like it, blow it away with package:implode."
      else
        cd 'ext' do
          %x{git clone #{@packaging_url}}
        end
      end
    end
    desc "Remove all cloned packaging automation"
    task :implode do
      rm_rf "./ext/#{@packaging_repo}"
    end
  end
end

## Helpers for compiling assets
###############################

def js(name, *deps)
  task name => File.join(STATIC_CONTENT, 'js') do
    File.open(File.join(STATIC_CONTENT, 'js', "#{name}.js"), 'w+') do |f|
      deps.each do |js_file|
        f.puts File.read(File.join(RAKE_ROOT, 'assets', 'js', js_file))
      end
    end
  end
end

def sass(name, *deps)
  task name => File.join(STATIC_CONTENT, 'css') do
    require 'sass'
    Sass.load_paths.clear
    Sass.load_paths << "assets/scss"

    File.open(File.join(STATIC_CONTENT, 'css', "#{name}.css"), 'w+') do |f|
      deps.each do |sass|
        f.puts Sass.compile_file(File.join(RAKE_ROOT, 'assets', 'scss', sass))
      end
    end
  end
end
